# coding: utf-8

# flake8: noqa
"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from istari_digital_client.models.access_relation import AccessRelation
from istari_digital_client.models.access_relationship import AccessRelationship
from istari_digital_client.models.access_resource_type import AccessResourceType
from istari_digital_client.models.access_subject_type import AccessSubjectType
from istari_digital_client.models.agent import Agent
from istari_digital_client.models.agent_display_name import AgentDisplayName
from istari_digital_client.models.agent_information import AgentInformation
from istari_digital_client.models.agent_module_version import AgentModuleVersion
from istari_digital_client.models.agent_modules import AgentModules
from istari_digital_client.models.agent_status import AgentStatus
from istari_digital_client.models.agent_status_name import AgentStatusName
from istari_digital_client.models.archive import Archive
from istari_digital_client.models.archive_status import ArchiveStatus
from istari_digital_client.models.archive_status_name import ArchiveStatusName
from istari_digital_client.models.artifact import Artifact
from istari_digital_client.models.check_result import CheckResult
from istari_digital_client.models.comment import Comment
from istari_digital_client.models.control_tag import ControlTag
from istari_digital_client.models.control_tag_revision import ControlTagRevision
from istari_digital_client.models.control_tag_status import ControlTagStatus
from istari_digital_client.models.control_tagging_object_type import ControlTaggingObjectType
from istari_digital_client.models.control_tagging_status import ControlTaggingStatus
from istari_digital_client.models.deprecation_reason import DeprecationReason
from istari_digital_client.models.dry_run_snapshot import DryRunSnapshot
from istari_digital_client.models.file import File
from istari_digital_client.models.file_archive_status import FileArchiveStatus
from istari_digital_client.models.file_control_tagging import FileControlTagging
from istari_digital_client.models.file_revision import FileRevision
from istari_digital_client.models.file_revision_archive_status import FileRevisionArchiveStatus
from istari_digital_client.models.filter_by import FilterBy
from istari_digital_client.models.function import Function
from istari_digital_client.models.function_auth_provider import FunctionAuthProvider
from istari_digital_client.models.function_auth_provider_update import FunctionAuthProviderUpdate
from istari_digital_client.models.function_auth_secret import FunctionAuthSecret
from istari_digital_client.models.function_auth_type import FunctionAuthType
from istari_digital_client.models.function_schema import FunctionSchema
from istari_digital_client.models.function_status import FunctionStatus
from istari_digital_client.models.http_validation_error import HTTPValidationError
from istari_digital_client.models.healthcheck_report import HealthcheckReport
from istari_digital_client.models.input_schema import InputSchema
from istari_digital_client.models.input_type import InputType
from istari_digital_client.models.job import Job
from istari_digital_client.models.job_status import JobStatus
from istari_digital_client.models.job_status_name import JobStatusName
from istari_digital_client.models.model import Model
from istari_digital_client.models.model_list_item import ModelListItem
from istari_digital_client.models.module import Module
from istari_digital_client.models.module_author import ModuleAuthor
from istari_digital_client.models.module_author_manifest import ModuleAuthorManifest
from istari_digital_client.models.module_type import ModuleType
from istari_digital_client.models.module_version import ModuleVersion
from istari_digital_client.models.module_version_status import ModuleVersionStatus
from istari_digital_client.models.new_agent import NewAgent
from istari_digital_client.models.new_agent_display_name import NewAgentDisplayName
from istari_digital_client.models.new_agent_information import NewAgentInformation
from istari_digital_client.models.new_agent_module_version import NewAgentModuleVersion
from istari_digital_client.models.new_agent_status import NewAgentStatus
from istari_digital_client.models.new_control_tag import NewControlTag
from istari_digital_client.models.new_function_auth_secret import NewFunctionAuthSecret
from istari_digital_client.models.new_function_manifest import NewFunctionManifest
from istari_digital_client.models.new_function_schema_manifest import NewFunctionSchemaManifest
from istari_digital_client.models.new_input_schema_manifest import NewInputSchemaManifest
from istari_digital_client.models.new_job_assigned_agent import NewJobAssignedAgent
from istari_digital_client.models.new_module_author import NewModuleAuthor
from istari_digital_client.models.new_module_manifest import NewModuleManifest
from istari_digital_client.models.new_operating_system import NewOperatingSystem
from istari_digital_client.models.new_output_schema_manifest import NewOutputSchemaManifest
from istari_digital_client.models.new_snapshot import NewSnapshot
from istari_digital_client.models.new_snapshot_tag import NewSnapshotTag
from istari_digital_client.models.new_system import NewSystem
from istari_digital_client.models.new_system_configuration import NewSystemConfiguration
from istari_digital_client.models.new_tool import NewTool
from istari_digital_client.models.new_tool_version import NewToolVersion
from istari_digital_client.models.new_tracked_file import NewTrackedFile
from istari_digital_client.models.no_op_response import NoOpResponse
from istari_digital_client.models.operating_system import OperatingSystem
from istari_digital_client.models.output_schema import OutputSchema
from istari_digital_client.models.output_type import OutputType
from istari_digital_client.models.page_agent import PageAgent
from istari_digital_client.models.page_agent_status import PageAgentStatus
from istari_digital_client.models.page_artifact import PageArtifact
from istari_digital_client.models.page_comment import PageComment
from istari_digital_client.models.page_file import PageFile
from istari_digital_client.models.page_function import PageFunction
from istari_digital_client.models.page_function_auth_provider import PageFunctionAuthProvider
from istari_digital_client.models.page_job import PageJob
from istari_digital_client.models.page_model_list_item import PageModelListItem
from istari_digital_client.models.page_module import PageModule
from istari_digital_client.models.page_module_author import PageModuleAuthor
from istari_digital_client.models.page_module_version import PageModuleVersion
from istari_digital_client.models.page_operating_system import PageOperatingSystem
from istari_digital_client.models.page_personal_access_token import PagePersonalAccessToken
from istari_digital_client.models.page_resource_search_item import PageResourceSearchItem
from istari_digital_client.models.page_snapshot import PageSnapshot
from istari_digital_client.models.page_snapshot_item import PageSnapshotItem
from istari_digital_client.models.page_snapshot_revision_search_item import PageSnapshotRevisionSearchItem
from istari_digital_client.models.page_snapshot_tag import PageSnapshotTag
from istari_digital_client.models.page_system import PageSystem
from istari_digital_client.models.page_system_configuration import PageSystemConfiguration
from istari_digital_client.models.page_tool import PageTool
from istari_digital_client.models.page_tool_version import PageToolVersion
from istari_digital_client.models.page_tracked_file import PageTrackedFile
from istari_digital_client.models.patch_op import PatchOp
from istari_digital_client.models.personal_access_token import PersonalAccessToken
from istari_digital_client.models.product import Product
from istari_digital_client.models.resource_archive_status import ResourceArchiveStatus
from istari_digital_client.models.resource_control_tagging import ResourceControlTagging
from istari_digital_client.models.resource_search_item import ResourceSearchItem
from istari_digital_client.models.resource_type import ResourceType
from istari_digital_client.models.response_create_snapshot import ResponseCreateSnapshot
from istari_digital_client.models.restore import Restore
from istari_digital_client.models.snapshot import Snapshot
from istari_digital_client.models.snapshot_item import SnapshotItem
from istari_digital_client.models.snapshot_revision_search_item import SnapshotRevisionSearchItem
from istari_digital_client.models.snapshot_tag import SnapshotTag
from istari_digital_client.models.source import Source
from istari_digital_client.models.subject_info import SubjectInfo
from istari_digital_client.models.system import System
from istari_digital_client.models.system_baseline import SystemBaseline
from istari_digital_client.models.system_configuration import SystemConfiguration
from istari_digital_client.models.tenant_public_key import TenantPublicKey
from istari_digital_client.models.token import Token
from istari_digital_client.models.token_with_properties import TokenWithProperties
from istari_digital_client.models.tool import Tool
from istari_digital_client.models.tool_version import ToolVersion
from istari_digital_client.models.tracked_file import TrackedFile
from istari_digital_client.models.tracked_file_specifier_type import TrackedFileSpecifierType
from istari_digital_client.models.update_access_relationship import UpdateAccessRelationship
from istari_digital_client.models.update_control_tag import UpdateControlTag
from istari_digital_client.models.update_system import UpdateSystem
from istari_digital_client.models.update_tag import UpdateTag
from istari_digital_client.models.update_tool import UpdateTool
from istari_digital_client.models.upload_as import UploadAs
from istari_digital_client.models.usability_status_name import UsabilityStatusName
from istari_digital_client.models.usability_status_params import UsabilityStatusParams
from istari_digital_client.models.user import User
from istari_digital_client.models.user_control_tagging import UserControlTagging
from istari_digital_client.models.user_model_inputs import UserModelInputs
from istari_digital_client.models.user_state import UserState
from istari_digital_client.models.user_state_option import UserStateOption
from istari_digital_client.models.user_type import UserType
from istari_digital_client.models.validation_error import ValidationError
from istari_digital_client.models.validation_error_loc_inner import ValidationErrorLocInner
from istari_digital_client.models.new_source import NewSource
from istari_digital_client.models.properties import Properties
