# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class FunctionAuthType(str, Enum):
    """
    FunctionAuthType
    """

    """
    allowed enum values
    """
    JWT = 'JWT'
    TOKEN = 'TOKEN'
    BASIC = 'BASIC'
    OAUTH1 = 'OAUTH1'
    OAUTH2 = 'OAUTH2'
    CAS_DELEGATED_3_DX = 'CAS_DELEGATED_3DX'
    OAUTH1_REGISTRATION = 'OAUTH1_REGISTRATION'
    OAUTH2_REGISTRATION = 'OAUTH2_REGISTRATION'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FunctionAuthType from a JSON string"""
        return cls(json.loads(json_str))
