# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AgentStatusName(str, Enum):
    """
    AgentStatusName
    """

    """
    allowed enum values
    """
    IDLE = 'Idle'
    CLAIMINGJOB = 'ClaimingJob'
    EXECUTINGJOB = 'ExecutingJob'
    EXECUTIONFAILED = 'ExecutionFailed'
    EXECUTIONSUCCESS = 'ExecutionSuccess'
    UPDATINGAGENT = 'UpdatingAgent'
    SUCCESSFULAGENTUPDATE = 'SuccessfulAgentUpdate'
    FAILEDAGENTUPDATE = 'FailedAgentUpdate'
    UPDATINGMODULE = 'UpdatingModule'
    SUCCESSFULMODULEUPDATE = 'SuccessfulModuleUpdate'
    FAILEDMODULEUPDATE = 'FailedModuleUpdate'
    PAUSED = 'Paused'
    UNKNOWN = 'Unknown'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgentStatusName from a JSON string"""
        return cls(json.loads(json_str))
